******************************************************************;
**************Chapter1. Scatterplots and Regression***************;
********Detailed SAS scripts including graphical settings*********;
******************************************************************;
*****************************alr{F1.1}****************************;
options nocenter linesize=75 pagesize=66;
proc gplot data=alr3.heights;
  plot Dheight*Mheight;
  run; quit;
goptions reset=all;
title 'Figure 1.1';
symbol v=dot c=black h=.1;
axis1 label=('Mheight');
axis2 label=(a=90 'Dheight');
proc gplot data=alr3.heights;
  plot Dheight*Mheight /haxis=axis1 haxis=55 to 75 hminor=0
                        vaxis=axis2 vaxis=55 to 75 vminor=0;
run; quit;

*****************************alr{F1.2}****************************;
goptions reset=all;
title ' ';
symbol v=dot c=black h=.1;
axis1 label=('Mheight');
axis2 label=(a=90 'Dheight');
proc gplot data=alr3.heights;
  plot Dheight*Mheight /haxis=axis1 haxis=55 to 75 hminor=0
                        vaxis=axis2 vaxis=55 to 75 vminor=0;
  where (Mheight ge 57.5) & (Mheight le 58.5) |
        (Mheight ge 62.5) & (Mheight le 63.5) |
        (Mheight ge 67.5) & (Mheight le 68.5);
run; quit; 

*****************************alr{F1.3}****************************;
goptions reset=all;
symbol v=circle h=1;
proc reg data=alr3.forbes;
  model Pressure=Temp;
  plot Pressure*Temp;
  plot residual.*Temp;
run;

*****************************alr{F1.4}****************************;
proc reg data=forbes;
  model Lpres=Temp;
  plot (Lpres residual.)*Temp;
run;

*****************************alr{F1.5}****************************;
**********E(Length|Age) by Age obtained using proc loess**********;
proc loess data=alr3.wblake;
  model Length=Age /smooth=.1;
  ods output OutputStatistics=m1;
run;
proc reg data=alr3.wblake;
  model Length=Age;
  output out=m2 pred=ols;
run;
data m3; set alr3.wblake; set m1; set m2;
proc sort data=m3; by Age;
goptions reset=all;
symbol1 v=circle h=1 c=black l=1 i=join;
symbol2 v=circle h=1 c=black l=2 i=join;
symbol3 v=circle h=1 c=black;
axis2 label=(a=90 'Length');
proc gplot data=m3;
  plot ols*Age=1 Pred*Age=2 Length*Age=3
       /overlay hminor=0 vaxis=axis2 vminor=0;
run; quit;

*****************************alr{F1.7}****************************;
goptions reset=all;
axis1 label=('Amount (percent of diet)');
axis2 label=(a=90 'Weight gain, g');
proc gplot data=alr3.turkey;
  plot gain*A=S /haxis=axis1 vaxis=axis2;
run; quit;

*****************************alr{F1.8}****************************;
proc reg data=alr3.heights;
  model Dheight=Mheight;
  output out=m1 predicted=Dhat;
run;
goptions reset=all;
symbol1 v=dot h=.1 l=1;
symbol2 v=dot h=.1 l=2;
symbol3 v=dot h=.1;
axis1 label=('Mheight');
axis2 label=(a=90 'Dheight');
proc gplot data=m1;
  plot Dhat*Mheight=1 Mheight*Mheight=2 Dheight*Mheight=3
       /overlay haxis=axis1 hminor=0 vaxis=axis2 vminor=0;
run; quit;

****************************alr{F1.10}****************************;
***the format statement in proc gplot modifies the scale on axes**;
proc reg data=alr3.heights;
  model Dheight=Mheight;
  output out=m1 predicted=Dhat;
run;
proc loess data=alr3.heights;
  model Dheight=Mheight /smooth=.6;
  ods output OutputStatistics=myout;
run;
data myout; set myout; set m1;
proc sort data=myout; by Mheight; run;
goptions reset=all;
symbol1 v=dot h=.1 c=black;
symbol2 v=dot h=.1 c=black i=join l=3;
symbol3 v=dot h=.1 c=black i=join l=1;
axis1 label=('Mheight');
axis2 label=(a=90 'Dheight');
proc gplot data=myout;
  format DepVar f4.0 Mheight f8.0;
  plot Dhat*Mheight=3 Pred*Mheight=2 DepVar*Mheight=1
       /overlay haxis=axis1 hminor=0 vaxis=axis2 vminor=0;
run; quit;

****************************alr{F1.11}****************************;
data m1;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  logMiles=log2(Miles);
goptions reset=all;
proc insight data=m1;
  scatter Tax Dlic income logMiles Fuel
         *Tax Dlic income logMiles Fuel;
run;
