******************************************************************;
******************Chapter10. Variable Selection*******************;
******************************************************************;

************OLS regression model for simulated data 1*************;
************************independent predictors********************;
options nocenter ls=75;
data case1;
  seed=20040622;
  do i=1 to 100;
    x1=rannor(seed);
    x2=rannor(seed);
    x3=rannor(seed);
    x4=rannor(seed);
    e=rannor(seed);
    y=1+x1+x2+e;
    output;
  end;
proc print data=case1; where (i le 3 | i ge 98); run;
proc reg data=case1;
  model y=x1-x4;
run;

************OLS regression model for simulated data 2*************;
************************correlated predictors*********************;
data temp (drop=y); set case1;
proc iml;
use temp;
read all var {x1 x2 x3 x4} into x;
read all var {e} into e;
SIGMA={  1     0  .95     0,
         0     1    0  -.95,
       .95     0    1     0,
         0  -.95    0     1};
r=root(SIGMA);
x=x*r; *each row of this x has a 4-dim;
       *multivariate normal distribution;
       *with mean 0 and variance SIGMA;
x1=x[,1]; x2=x[,2]; x3=x[,3]; x4=x[,4];
create case2 var {x1 x2 x3 x4 e};
append var {x1 x2 x3 x4 e};
close case2;
quit;
data case2; set case2; y=x1+x2+e;
proc reg data=case2;
  model y=x1-x4;
run;

*****displaying Variance Inflation Factors for simulated data*****;
proc reg data=case1;
  model y=x1-x4 /vif;
run;
proc reg data=case2;
  model y=x1-x4 /vif;
run;

*******displaying model fitting information (AIC, BIC, etc.)******;
*****for both full model and a submodel with the highway data*****;
data highway;
  set alr3.highway;
  logLen=log2(Len); logADT=log2(ADT);
  logTrks=log2(Trks); logSigs1=log2((Len*Sigs+1)/Len);
  if (Hwy eq 1) then Hwy1=1; else Hwy1=0; *create dummies for variable Hwy;
  if (Hwy eq 2) then Hwy2=1; else Hwy2=0;
  if (Hwy eq 3) then Hwy3=1; else Hwy3=0;
  logRate=log2(Rate);
proc reg data=highway outest=m1
  (keep=_model_ _aic_ _bic_ _cp_ _press_ _p_ _sse_ _mse_ _sbc_);
  fullmodel: model logRate=logADT logTrks Lane Acpt logSigs1 Itg Slim
                logLen Lwid Shld Hwy1 Hwy2 Hwy3
                /noprint aic bic cp press sse mse sbc;
  submodel: model logRate=logLen Slim Acpt logTrks Shld
                /noprint aic bic cp press sse mse sbc;
run;
goptions reset=all;
title 'Model fit and diagnostic statistics for the highway data';
proc print data=m1; run; *print some statistics for the full model, as;
                         *in alr{10.2.1}. BIC as in alr{E10.7} is displayed;
                         *as SBC. And Cp for the submodel is incorrect;

******************submodel selection based on Cp******************;
goptions reset=all;
data highway;
  set alr3.highway;
  logLen=log2(Len); logADT=log2(ADT);
  logTrks=log2(Trks); logSigs1=log2((Len*Sigs+1)/Len);
  if (Hwy eq 1) then Hwy1=1; else Hwy1=0;
  if (Hwy eq 2) then Hwy2=1; else Hwy2=0;
  if (Hwy eq 3) then Hwy3=1; else Hwy3=0;
  logRate=log2(Rate);
*********************************************************************;
****If you use Cp, you need to specify sigma (from the full model)***;
****You can tell sas what the estimated sigma is, via /sigma=value***;
*You can also tell sas to display the best n (say 30) submodels only*;
*********************************************************************;
options ls=70;
proc reg data=highway;
  model logRate=logADT logTrks Lane Acpt logSigs1 Itg Slim
                logLen Lwid Shld Hwy1 Hwy2 Hwy3
                /selection=cp aic best=30;
run; quit;

*********************************************************************;
**********************forward submodel selection*********************;
*********************************************************************;
proc reg data=highway;
  model logRate=logADT logTrks Lane Acpt logSigs1 Itg Slim
                logLen Lwid Shld Hwy1 Hwy2 Hwy3
                /selection=forward;
  ods output SelectionSummary=submodels;
run; quit;
proc print data=submodels; run;
