******************************************************************;
*****************Chapter11. Nonlinear regression******************;
******************************************************************;

******nonlinear model with proc nlmixed for the turk0 data********;
options nocenter ls=78;
proc nlmixed data=alr3.turk0 corr;
  parms th1=620 th2=200 th3=10 sig2=10;
  fitGain=th1+th2*(1-exp(-th3*A));
  model Gain~Normal(fitGain,sig2);
  predict fitGain out=ghat;
run; quit;
goptions reset=all;
symbol1 v=circle c=black;
symbol2 v=point c=black i=join l=1;
proc gplot data=ghat;
  plot Gain*A=1 Pred*A=2 /overlay hminor=0 vminor=0;
run; quit;

*****lack of fit test for nonlinear model with the turk0 data*****;
goptions reset=all;
proc nlmixed data=alr3.turk0;
  parms th1=620 th2=200 th3=10 sig2=10;
  fitGain=th1+th2*(1-exp(-th3*A));
  model Gain~Normal(fitGain,sig2);
  ods output ParameterEstimates=a1;
run; quit; 
data a1; set a1; where (Parameter='sig2'); SS=Estimate*DF; DF=32;
proc glm data=turk0;
  class A;
  model Gain=A;
  ods output OverallAnova=a2;
run; quit;
data a2; set a2; where (Source='Error');
data lof (keep=DF SS); set a1 a2;
proc sort data=lof; by descending DF; 
proc iml;
use lof;
read all var {DF} into resDF;
read all var {SS} into resSS;
DF=j(2,1,.); Sumsq=j(2,1,.); F=j(2,1,.); pval=j(2,1,.);
DF[2]=resDF[1]-resDF[2];
Sumsq[2]=resSS[1]-resSS[2];
F[2]=Sumsq[2]/DF[2]/(resSS[2]/resDF[2]);
pval[2]=1-probF(F[2],DF[2],resDF[2]);
title 'lack of fit test';
print resDF resSS DF Sumsq F pval;
quit;

*********ANOVA for nonlinear models with the turkey data**********;
goptions reset=all;
data turkey;
  set alr3.turkey;
  S1=(S eq 1);
  S2=(S eq 2);
  S3=(S eq 3);
  w=sqrt(m);
  wGain=w*Gain;
proc nlin data=turkey; *most general;
  parms th11=620 th12=620 th13=620
th21=200 th22=200 th23=200
th31=10 th32=10 th33=10;
  model wGain=w*(S1*(th11+th21*(1-exp(-th31*A)))+
S2*(th12+th22*(1-exp(-th32*A)))+
S3*(th13+th23*(1-exp(-th33*A))));
  ods output Anova=a1;
run; quit;
proc nlin data=turkey; *common intercept;
  parms th1=620
th21=200 th22=200 th23=200
th31=10 th32=10 th33=10;
  model wGain=w*(th1+
S1*(th21*(1-exp(-th31*A)))+
S2*(th22*(1-exp(-th32*A)))+
S3*(th23*(1-exp(-th33*A))));
  ods output Anova=a2;
run; quit;
proc nlin data=turkey; *common intercept and asymptote;
  parms th1=620 th2=200
th31=10 th32=10 th33=10;
  model wGain=w*(th1+th2*(
S1*(1-exp(-th31*A))+
S2*(1-exp(-th32*A))+
S3*(1-exp(-th33*A))));
  ods output Anova=a3;
run; quit;
proc nlin data=turkey; *common regression;
  parms th1=620 th2=200 th3=10;
  model wGain=w*(th1+th2*(1-exp(-th3*A)));
  ods output Anova=a4;
run; quit;
**************************************;
***Anova table for model 1, 2 and 4***;
**************************************;
data anova (keep=DF SS); set a1 a2 a4; where (Source eq 'Residual');
proc sort data=anova; by descending DF;
proc iml;
use anova;
read all var {DF} into resDF;
read all var {SS} into resSS;
DF=j(3,1,.); Sumsq=j(3,1,.); F=j(3,1,.); pval=j(3,1,.);
DF[2]=resDF[1]-resDF[2]; DF[3]=resDF[2]-resDF[3];
Sumsq[2]=resSS[1]-resSS[2]; Sumsq[3]=resSS[2]-resSS[3];
F[2]=Sumsq[2]/DF[2]/(resSS[2]/resDF[2]);
F[3]=Sumsq[3]/DF[3]/(resSS[3]/resDF[3]);
pval[2]=1-probF(F[2],DF[2],resDF[2]);
pval[3]=1-probF(F[3],DF[3],resDF[3]);
title 'Analysis of Variance Table';
print resDF resSS DF Sumsq F pval;
quit;
**************************************;
***Anova table for model 1, 3 and 4***;
**************************************;
data anova (keep=DF SS); set a1 a3 a4; where (Source eq 'Residual');
proc sort data=anova; by descending DF;
proc iml;
use anova;
read all var {DF} into resDF;
read all var {SS} into resSS;
DF=j(3,1,.); Sumsq=j(3,1,.); F=j(3,1,.); pval=j(3,1,.);
DF[2]=resDF[1]-resDF[2]; DF[3]=resDF[2]-resDF[3];
Sumsq[2]=resSS[1]-resSS[2]; Sumsq[3]=resSS[2]-resSS[3];
F[2]=Sumsq[2]/DF[2]/(resSS[2]/resDF[2]);
F[3]=Sumsq[3]/DF[3]/(resSS[3]/resDF[3]);
pval[2]=1-probF(F[2],DF[2],resDF[2]);
pval[3]=1-probF(F[3],DF[3],resDF[3]);
title 'Analysis of Variance Table';
print resDF resSS DF Sumsq F pval;
quit;

******bootstrap for nonlinear regression with the segreg data*****;
*******************************************; 
***A similar macro appeared in chapter 4***;
*******************************************;
%let n=39; *#obs in the data;
%macro nlsboot(B=1, seed=1234, outputdata=temp);
%do i=1 %to &B;
*the following command tells SAS to clear both the; 
*log window and the output window;
dm 'log; clear; output; clear; ';
data analysis;
  choice=int(ranuni(&seed*&i)*&n)+1;
  set alr3.segreg point=choice;
  j+1;
  if j>&n then STOP;
proc nlmixed data=analysis;
  parms th0=70 th1=.5 gamma=40 sig2=1;
  fitC=th0+th1*max(0,Temp-gamma);
  model C~Normal(fitC,sig2);
  ods output ParameterEstimates=pe;
run; quit;
proc sql; create table outests as (select Estimate, 
  sum(Estimate*(Parameter='th0')) as th0, 
  sum(Estimate*(Parameter='th1')) as th1, 
  sum(Estimate*(Parameter='gamma')) as gamma from pe); 
quit;
data outests (keep=th0 th1 gamma); set outests (obs=1);
proc append base=&outputdata data=outests; run;
%end;
%mend nlsboot;
***example as in the book, with B=999 resample estimates***;
%nlsboot(B=999, seed=2004, outputdata=mynlboot);
proc univariate data=mynlboot noprint;
  var th0 th1 gamma;
  output out=mynlbootstat mean=mean_th0 mean_th1 mean_gamma
         std=sd_th0 sd_th1 sd_gamma
         pctlpts=2.5, 97.5 pctlpre=th0 th1 gamma;
run;
goptions reset=all;
proc iml; *we launch proc iml to have a formatted output;
use mynlbootstat;
read all var _all_ into summary;
out=j(4,3,.);
out[1,]=t(summary[1:3]);
out[2,]=t(summary[4:6]);
out[3,]=t(summary[{7,9,11}]);
out[4,]=t(summary[{8,10,12}]);
print out[rowname={'Mean' 'SD' '2.5%' '97.5%'}
          colname={'th0' 'th1' 'gamma'} label=""];
quit;
