******************************************************************;
******************Chapter12. Logistic Regression******************;
******************************************************************;

*********example of SAS proc genmod with the blowBF data**********;
options nocenter ls=75;
data blowBF;
  set alr3.blowBF;
  logD=log2(D);
proc genmod data=blowBF;
  model y=logD /dist=binomial link=logit;
run; quit;

*******testing contrast in proc genmod with the blowBF data*******;
goptions reset=all;
options linesize=90;
proc genmod data=alr3.titanic;
  class Class Age Sex;
  model Surv/N=Class Age Sex / dist=binomial link=logit;
  contrast "Class crew?=Class first" Class 1 -1 0 0 /wald;
  contrast "Class second?=Class third" Class 0 0 1 -1 /wald;
  contrast "Female?=Male" Sex 1 -1 /wald;
run;

*****comparison of OLS, logistic and OLS with the blowBF data*****;
data jitter;
  set alr3.blowBF;
  logD=log2(D);
  logD_j=logD+.1*(ranuni(-1)-0.5); *add some symmetric noise to data;
  y_j=y+.02*(ranuni(-1)-0.5);
goptions reset=all;
symbol1 v=circle h=1 c=black;
symbol2 v=point c=black i=join l=1;
symbol3 v=point c=black i=join l=2;
symbol4 v=point c=black i=join l=3;
axis1 label=('log2(Diameter)');
axis2 label=(a=90 'Blowdown indicator');
legend1 label=none value=(h=1 'y' 'OLS' 'Logit' 'Lowess')
        position=(top left inside) mode=protect cborder=black;
proc reg data=blowBF;
  model y=logD;
  output out=m1 predicted=pred_ols;
run;
proc genmod descending data=blowBF;
  model y=logD /dist=binomial link=logit;
  output out=m2 p=pred_logit;
run; quit;
proc loess data=blowBF;
    model y=logD /smooth=.6;
    ods output OutputStatistics=m3;
run; quit;
data combine; set jitter; set m1; set m2; set m3;
proc sort data=combine; by logD;
proc gplot data=combine;
  plot y_j*logD_j=1 pred_ols*logD=2 pred_logit*logD=3 Pred*logD=4
       /overlay haxis=axis1 hminor=0 vaxis=axis2 vminor=0 legend=legend1;
run; quit;

****************density estimation (alr{F12.1b})******************;
goptions reset=all;
data blowBF;
  set alr3.blowBF;
  logD=log2(D);
proc kde data=blowBF (where=(y=0)) bwm=.7 out=KDEOut0;
  var logD;
quit;
proc kde data=blowBF (where=(y=1)) bwm=.7 out=KDEOut1;
  var logD;
quit;
data KDEOut;
set KDEOut0; density0=density;
set KDEOut1; density1=density;
symbol1 v=point c=black i=join l=1;
symbol2 v=point c=black i=join l=2;
axis2 label=(a=90 'Kernel density estimate');
legend1 label=none value=(h=1 'Y=0' 'Y=1')
position=(top right inside) cborder=black;
proc gplot data=KDEOut;
  plot density0*logD=1 density1*logD=2
/overlay legend=legend1 hminor=0 vaxis=axis2 vminor=0;
run; quit;

*************logistic models with multiple predictors*************;
goptions reset=all;
data blowBF;
  set alr3.blowBF;
  logD=log2(D);
proc genmod descending data=blowBF;
  model y=logD S /dist=binomial link=logit;
  output out=many1 p=pred_prob;
run; quit;
proc genmod descending data=blowBF;
  model y=logD S logD*S /dist=binomial link=logit;
  output out=many2 p=pred_prob;
run; quit;

***************plot for odds multiplier (alr{F12.5})**************;
data blowBF;
  set alr3.blowBF;
  logD=log2(D);
proc genmod descending data=blowBF;
  model y=logD S logD*S /dist=binomial link=logit;
  ods output ParameterEstimates=m3;
run; quit;
data temp1; set m3; where (Parameter='S');
call symput('coef3',trim(left(Estimate)));
data temp1; set m3; where (Parameter='logD*S');
call symput('coef4',trim(left(Estimate)));
proc iml;
use blowBF;
read all var {logD} into LD;
minLD=min(LD);
maxLD=max(LD);
logD=minLD+(0:99)*(maxLD-minLD)/99;
D=2**logD;
odm=exp(&coef3/10+&coef4*logD/10);
create tempdata var {D logD odm};
append var {D logD odm};
close tempdata;
quit;
goptions reset=all;
symbol v=point c=black i=join l=1;
axis2 label=(a=90 'Odds multiplier');
proc gplot data=tempdata;
  plot odm*D /haxis=5 to 35 by 5 hminor=0
vaxis=axis2 vminor=0;
run; quit;

**********ANOVA for logistic models with the blowBF data**********;
goptions reset=all;
data blowBF;
  set alr3.blowBF;
  logD=log2(D);
proc genmod descending data=blowBF;
  model y=logD / dist=binomial link=logit;
  ods output ModelFit=m1;
run; quit;
proc genmod descending data=blowBF;
  model y=logD S / dist=binomial link=logit;
  ods output ModelFit=m2;
run; quit;
proc genmod descending data=blowBF;
  model y=logD S logD*S/ dist=binomial link=logit;
  ods output ModelFit=m3;
run; quit;
data test (keep=DF value);
set m1 m2 m3; where (criterion eq 'Deviance');
proc iml;
use test;
read all var _all_ into x; *_all_ stands for all variables in data 'test';
resDF=x[,1]; resDev=x[,2];
Model={"m1", "m2", "m3"};
DF=j(3,1,.); Dev=j(3,1,.); pval=j(3,1,.);
DF[2]=resDF[1]-resDF[2]; DF[3]=resDF[2]-resDF[3];
Dev[2]=resDev[1]-resDev[2]; Dev[3]=resDev[2]-resDev[3];
pval[2]=1-probchi(Dev[2],DF[2]);
pval[3]=1-probchi(Dev[3],DF[3]);
title 'Analysis of Variance Table for model m1, m2 and m3';
print Model resDF resDev DF Dev pval;
quit;

***********assessing lack of fit with the blowBF data*************;
goptions reset=all;
data blowBF;
  set alr3.blowBF;
  logD=log2(D);
proc genmod descending data=blowBF;
  model y=logD / dist=binomial link=logit;
run; quit;
