******************************************************************;
***************Chapter2. Simple Linear Regression*****************;
******************************************************************;

**************summary statistics using proc means*****************;
options nocenter linesize=75 pagesize=66;
proc means data=alr3.forbes;
  var Temp Lpres;
run;

***************summary statistics using proc iml******************;
data f;
  set alr3.forbes ;
  one=1;
proc iml;
use f;
read all var {one Temp Pressure Lpres}
     where (Temp^=. & Pressure^=. & Lpres^=.) into X;
size=nrow(X); W=X`*X;
Tempbar=W[1,2]/size; Lpresbar=W[1,4]/size; Y=j(size,2,0);
Y[,1]=X[,2]-Tempbar; Y[,2]=X[,4]-Lpresbar;
fcov=Y`*Y; RSS=fcov[2,2]-fcov[1,2]**2/fcov[1,1];
sigmahat2=RSS/(size-2); sigmahat=sqrt(sigmahat2);
b1=fcov[1,2]/fcov[1,1];
b0=Lpresbar-b1*Tempbar;
title 'PROC IML results';
print Tempbar Lpresbar fcov b1 b0 RSS sigmahat2;
create forbes_stat var{Tempbar Lpresbar fcov b1 b0 RSS sigmahat2};
append var{Tempbar Lpresbar fcov b1 b0 RSS sigmahat2};
close forbes_stat;
quit;

*********************ANOVA table and proc reg*********************;
proc reg data=alr3.forbes;
  model Lpres=Temp/I XPX R;
run;

**********confidence intervals for parameter estimates************;
proc reg data=alr3.forbes;
  model Lpres=Temp /clb alpha=.05;
run;

***********confidence intervals for each fitted value*************;
proc reg data=alr3.forbes alpha=.05;
  model Lpres=Temp /cli;
run;

*******confidence intervals for predictions using proc reg********;
data forbes2;
  input Temp;
cards;
210
220
;
data forbes2; *combine data sets;
  set alr3.forbes forbes2;
proc reg data=forbes2 alpha=.05;
  model Lpres=Temp /cli;
  output out=m1 predicted=prediction L95=Lower_limit U95=Upper_limit;
run;

*******confidence intervals for predictions using proc glm********;
proc glm data=alr3.forbes alpha=.05;
  model Lpres=Temp /cli;
  estimate 'Temp at 210' Intercept 1 Temp 210; 
  estimate 'Temp at 220' Intercept 1 Temp 220;
run; quit;

**************************the residuals***************************;
goptions reset=all;
symbol v=circle h=1;
proc reg data=alr3.forbes;
  model Lpres=Temp;
  output out=m1 residual=res predicted=pred;
  plot residual.*predicted. /nostat nomodel; *the residual plot;
run;
proc print data=work.m1 ;
run;
