******************************************************************;
******************Chapter3. Multiple Regression*******************;
******************************************************************;
****************2-dimensional added variable plot*****************;
data fuel;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  Income=Income/1000;
  logMiles=log2(Miles);
goptions reset=all;
title 'Added Variable Plot for Tax';
proc reg data=fuel;
  model Fuel Tax=Dlic Income logMiles;
  output out=m1 residual=res1 res2;
run;
proc reg data=m1;
  model res1=res2;
  plot res1*res2;
run;
proc reg data=fuel;
  model Fuel =Tax Dlic Income logMiles/partial;
  output out=m1 residual=res1 res2;
run;

***********************summary statistics*************************;
data fuel;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  Income=Income/1000;
  logMiles=log2(Miles);
goptions reset=all;
proc means data=fuel;
  var Tax Dlic Income logMiles Fuel;
run;

**manual computation of sample covariance and sample correlation**;
data fuel;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  Income=Income/1000;
  logMiles=log2(Miles);
goptions reset=all;
proc iml;
use fuel;
read all var {Tax Dlic Income logMiles Fuel} into X;
size=nrow(X); one=j(1,size,1);
ave=one*X/size;
Xave=one`*ave;
W=X-Xave; Y=W`*W;
Xcov=Y/(size-1);
Xstd=sqrt(diag(Xcov)); Xstd_inv=inv(Xstd);
Xcor=Xstd_inv*Xcov*Xstd_inv;
print Xcor Xcov;
quit;


************manual computation of parameter estimates*************;
data fuel;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  Income=Income/1000;
  logMiles=log2(Miles);
  one=1;
proc iml;
use fuel;
read all var {one Tax Dlic Income logMiles} where (one^=. &
     Tax^=. & Dlic^=. & Income^=. & logMiles^=. & fuel^=.) into X;
read all var 'fuel' where (one^=. &
     Tax^=. & Dlic^=. & Income^=. & logMiles^=. & fuel^=.) into Y;
beta_hat=inv(t(X)*X)*t(X)*Y;
print beta_hat;
quit;

********************multiple linear regression********************;
proc reg data=fuel;
  model Fuel=Tax Dlic Income logMiles;
run;

**********************add/delete variable(s)**********************;
data fuel;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  Income=Income/1000;
  logMiles=log2(Miles);
options ls=60;
proc glm data=fuel;
  model Fuel=Tax Dlic Income logMiles/ss1;
run;

*******************F test for model comparison********************;
data fuel;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  Income=Income/1000;
  logMiles=log2(Miles);
proc reg data=fuel;
  model Fuel=Tax Dlic Income logMiles;
  test Tax=0;
  test Dlic=Income=0;
run;

*****prediction with multiple regression model using proc reg*****;
data fuel;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  Income=Income/1000;
  logMiles=log2(Miles);
data fuel2;
  input Tax Dlic Income logMiles;
cards;
20 909 16.3 626
35 943 16.8 667
;
data fuel2; set fuel fuel2;
proc reg data=fuel2 alpha=.05;
  model Fuel=Tax Dlic Income logMiles /cli;
run;

*****prediction with multiple regression model using proc glm*****;
proc glm data=fuel alpha=.05;
  model Fuel=Tax Dlic Income logMiles /cli;
  estimate '1' Intercept 1 Tax 20 Dlic 909 Income 16.3 logMiles 626;
  estimate '2' Intercept 1 Tax 35 Dlic 943 Income 16.8 logMiles 667;
run; quit;

