******************************************************************;
******************Chapter4. Drawing conclusions*******************;
******************************************************************;

********************detection of aliased terms********************;
options nocenter linesize=70 pageno=1;
data BGS;
  set alr3.BGSgirls;
  DW9=WT9-WT2;
  DW18=WT18-WT9;
goptions reset=all;
proc reg data=BGS;
  model Soma=WT2 DW9 DW18 WT9 WT18;
run;

********************dealing with missing data*********************;
data sleep;
  set alr3.sleep1;
  logSWS=log(SWS);
  logBodyWt=log(BodyWt);
  logLife=log(Life);
  logGP=log(GP);
proc reg data=sleep;
  model logSWS=logBodyWt logLife logGP;
run;
proc reg data=sleep;
  model logSWS=logBodyWt logGP;
  var logLife;
run;

****simulation study for the transaction data using resampling****;
%let n=261; * sample size in transact data;
%macro boots(B=1, seed=1234, outputdata=temp);
%do i=1 %to &B; * repeat B times;
     data m1;   * data step to create data file 'm1'; 
       rownum=int(ranuni(&seed*&i)*&n)+1; * rownum = random row number;
       set alr3.transact point=rownum; * add row 'rownum' to data set 'm1';
       j+1;                            * increment row counter;
     if j>&n then STOP;                * stop with sample size is n;
* still in the loop, use proc reg to fit with bootstrap sample;
     proc reg data=m1 noprint outest=outests (keep=INTERCEPT T1 T2);
        model Time=T1 T2;
     run;  * keep coef estimates and then use proc append to save them;
     proc append base=&outputdata data=outests; run;
%end;  * end the do loop;
%mend boots; * end the macro
***example as in the book, with B=999 resample estimates***;
%boots(B=999, seed=2004, outputdata=bootout);
proc univariate data=work.bootout;
histogram;
run;
***summarize the 999 bootstraps more completely;
proc univariate data=bootout noprint;
  var INTERCEPT T1 T2;
  output out=bootstat mean=Inct_mean T1_mean T2_mean
         std=Inct_sd T1_sd T2_sd pctlpts=2.5, 97.5 pctlpre=Inct T1 T2;
run;
***print the bootstrap uncorrected means and standard errors***;
goptions reset=all;
proc print data=bootstat;
  var Inct_mean Inct_sd T1_mean T1_sd T2_mean T2_sd;
run;
***print the bootstrap confidence intervals***;
proc print data=bootstat;
  var Inct2_5 Inct97_5 T12_5 T197_5 T22_5 T297_5;
run;
***retrieve the OLS parameter estimates***;
proc reg data=alr3.transact noprint outest=trans_ols (keep=INTERCEPT T1 T2);
  model Time=T1 T2;
run;
***the observed mean of the 999 bootstrap estimates***;
data temp; set bootstat;
  call symput('INTERCEPT_boot',left(trim(Inct_mean)));
  call symput('T1_boot',left(trim(T1_mean)));
  call symput('T2_boot',left(trim(T2_mean)));
run;
***the parameter estimates from OLS regression***;
data temp; set trans_ols;
  call symput('INTERCEPT_ols',left(trim(INTERCEPT)));
  call symput('T1_ols',left(trim(T1)));
  call symput('T2_ols',left(trim(T2)));
run;
***HISTOGRAMS of bootstrap samples from the transaction data***;
proc univariate data=bootout noprint;
  histogram INTERCEPT /normal href=&INTERCEPT_ols &INTERCEPT_boot;
run;
proc univariate data=bootout noprint;
  histogram T1 /normal href=&T1_ols &T1_boot;
run;
proc univariate data=bootout noprint;
  histogram T2 /normal href=&T2_ols &T2_boot;
run;

****simulation study for the npdata data by adding random noise***;
%macro boots2(B=1, seed=1234, outputdata=temp2);
%do i=1 %to &B;
data analysis;
  set alr3.npdata;
  rerr1=rannor(&seed*&i)*SECPUE;
  rerr2=rannor(&seed*&i*(&i+21))*SEdens;
Y=CPUE+rerr1;
X=Density+rerr2;
proc reg data=analysis noprint outest=outests (keep=X);
model Y=X /noint;
run;
proc append base=&outputdata data=outests; run;
%end;
proc univariate data=&outputdata noprint;
  output out=boot2stat mean=beta_mean
         pctlpts=2.5, 97.5 pctlpre=beta;
run;
proc print data=boot2stat; run;
%mend boots2;

***example as in the book, with B=999 resample estimates***;
%boots2(B=999, seed=2004, outputdata=supp4boot2);
