******************************************************************;
************Chapter5. Weights, lack of fit, and more**************;
******************************************************************;

*******************weighted least squares model*******************;
options nocenter linesize=75 nopages=1;
data phy;
  set alr3.physics;
  w=1/SD**2;
proc reg data=phy;
  model y=x;
  weight w;
run;

***********prediction with weighted least squares model***********;
data phy2;
  input x w;
cards;
.1 .04
;
data phy2; set phy phy2;
proc reg data=phy2;
  model y=x /cli;
  weight w;
run;

**************polynomial weighted least squares model*************;
data phy;
  set alr3.physics;
  w=1/SD**2;
proc glm data=phy;
  model y=x x*x;
  weight w;
run; quit;

******prediction from polynomial weighted least squares model*****;
data phy;
  set alr3.physics;
  input x y SD;
  w=1/SD**2;
data phy2;
  input x w;
cards;
.1 .04
;
data phy2; set phy phy2;
proc glm data=phy2;
  model y=x x*x /predicted;
  weight w;
  output out=m2 predicted=ypred2;
run; quit;

**************scatterplot with linear regression fit**************;
******************and quadratic regression fit********************;
data phy;
  set alr3.physics;
  w=1/SD**2;
proc iml;
x=do(.05,.35,(.35-.05)/(50-1));
  *generate 50 new observations of x from .05 to .35;
create phy2 var{x};
append var{x};
  *create and append statements are used to save the new data;
close phy2; quit;
data phy2; set phy phy2;
  *set statement appends the second data set to the first one;
proc glm data=phy2;
  model y=x; *simple linear regression model;
  weight w;
  output out=m1 predicted=ypred1;
run; quit;
proc glm data=phy2;
  model y=x x*x /predicted; *polynomial of degree 2;
  weight w;
  output out=m2 predicted=ypred2;
run; quit;
data m1;
  merge m1 m2; *merge statement is similar to set statement;
proc sort data=m1; by x;
goption reset=all;
symbol1 v=circle c=black h=1;
symbol2 v=point c=black h=1 i=join l=1;
symbol3 v=point c=black h=1 i=join l=33;
axis1 label=('x=s^(-1/2)');
axis2 label=(a=90 'Cross section, y');
proc gplot data=m1;
  plot y*x=1 ypred1*x=2 ypred2*x=3
       /overlay haxis=axis1 hminor=0 vaxis=axis2 vminor=0;
run; quit;

****************lack of fit test for the fake data****************;
data temp;
  input x y;
  x2=x;
cards;
1 2.55
1 2.75
1 2.57
2 2.40
3 4.19
3 4.70
4 3.81
4 4.87
4 2.93
4 4.52
;
run;
options nocenter ls=75 pageno=1;
proc glm data=temp;
  class x2;
  model y=x x2;
run; quit;

***************general F-test for the fuel2001 data***************;
********************comparison of nested models*******************;
data fuel;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  Income=Income/1000;
  logMiles=log2(Miles);
proc reg data=fuel;
  model Fuel=Tax Dlic Income logMiles;
  test Dlic=Income=0;
  test Dlic=Income=Tax=0;
run;
proc glm data=fuel;
  model Fuel=Tax Dlic Income logMiles/e e2;
run;

************joint confidence region for the UN2 data**************;
proc reg data=alr3.UN2 alpha=.05 outest=m1 tableout covout;
  model logFertility=logPPgdp Purban /noprint;
run;
proc iml;
use m1;
read all var {_RMSE_ logPPgdp Purban} into X;
sigmahat2=X[1,1]**2; *extract the variance estimate;
X=X[,2:3];
beta1=X[1,1]; *extract the coefficient estimate;
beta2=X[1,2]; *extract the coefficient estimate;
se=X[2,];
cov=X[8:9,]; *extract the covariance;
invcov=inv(cov); *the inverse of the covariance matrix;
z_val=probit(.975)*{1 -1};
int1=beta1+z_val*se[1];
int2=beta2+z_val*se[2];
one=j(1,2,1);
f_val=2*finv(.95,2,49); *the cutoff value for the ellipse;
call eigen(V,E,invcov);
t=((1:1000)-1)#atan(1)#8/(1000-1);
a1=cos(t)*sqrt(f_val/V[1]);
a2=sin(t)*sqrt(f_val/V[2]);
invE=inv(E);
b1=invE[1,1]*a1+invE[1,2]*a2+beta1;
b2=invE[2,1]*a1+invE[2,2]*a2+beta2;
beta1=beta1*one;
beta2=beta2*one;
create cr var{b1 b2 int1 int2 beta1 beta2};
append var{b1 b2 int1 int2 beta1 beta2};
close cr;
quit;
goptions reset=all;
symbol v=point i=join l=1;
axis1 label=('Coefficient for log(PPgdp)');
axis2 label=(a=90 'Coefficient for Purban');
proc gplot data=cr;
  plot b2*b1 beta2*int1 int2*beta1
       /overlay haxis=axis1 hminor=0 vaxis=axis2 vminor=0;
run; quit;
