******************************************************************;
***************Chapter6. Polynomials and factors******************;
******************************************************************;

**********************polynomial regression***********************;
options nocenter linesize=75;
data jitter;
  set alr3.cakes;
  X1=X1+.2*(ranuni(-1)-0.5); *add some noise to data;
  X2=X2+.5*(ranuni(-1)-0.5);
goptions reset=all;
symbol v=circle h=1; 
proc gplot data=jitter;
  plot X2*X1 /hminor=0 vminor=0;
run; quit;

**********polynomial regression with several predictors***********;
proc glm data=alr3.cakes;
  model Y=X1 X1*X1 X2 X2*X2 X1*X2;
run; quit;

***************************alr{F6.3a}*****************************;
proc iml; *create new data;
x1=j(3,50,.); x2=j(1,150,.);
x1[1,]=32+(0:49)*(38-32)/49; x2[1:50]=j(1,50,340);
x1[2,]=32+(0:49)*(38-32)/49; x2[51:100]=j(1,50,350);
x1[3,]=32+(0:49)*(38-32)/49; x2[101:150]=j(1,50,360);
create newdata var {x1 x2};
append var {x1 x2};
close newdata; quit;
data cakes; set newdata alr3.cakes;
proc glm data=cakes;
  model Y=X1 X1*X1 X2 X2*X2 X1*X2 /predicted;
  output out=m1 predicted=yhat; *save the predicted values;
run; quit;
symbol v=point i=join l=1;
proc gplot data=m1; where Y eq .; *choose the new data only;
  plot yhat*x1=x2
       /haxis=(32 to 38 by 1) hminor=0
        vaxis=(3 to 9 by 1) vminor=0;
run; quit;

******************delta method for the cakes data*****************;
data cakes;
  set alr3.cakes;
  X2sq=X2**2;
proc reg data=cakes outest=m1 covout; *retrieve covariance matrix;
  model Y=X2 X2sq;
run;
proc iml; *compute se based on delta method;
use m1;
read all var {X2 X2sq} into X;
b=X[1,];
cov=X[3:4,];
est=-b[1]/(2*b[2]);
db1=-1/(2*b[2]);
db2=b[1]/(2*b[2]**2);
se=sqrt(cov[1,1]*db1*db1+2*cov[1,2]*db1*db2+cov[2,2]*db2*db2);
print "-b1/(2*b2)", est [format=10.4];
print "StdErr (based on delta method)", se [format=10.4];
quit;
data cakes;
  set alr3.cakes;
  Z2 = (X2-350)/10;
proc nlmixed data = cakes;
  parms gam0-gam2=0 s2=1;
  yhat=gam0+gam1*Z2+gam2*Z2*Z2;
  model Y~normal(yhat,s2);
  estimate '350-5*gam1/gam2' 350-5*gam1/gam2;
run; 

***************************alr{T6.1a}*****************************;
proc glm data=alr3.sleep1;
  class D /order = 5 1 2 3 4;
  model TS=D /ss1 solution clparm;
 run;
* the following is an alternative method to getting ests, using contrasts;
ods trace on;
proc glm data=alr3.sleep1;
  class D;
  model TS=D /noint solution;
  contrast '1 vs 2' D -1 1 0 0 0;
  contrast '1 vs 3' D -1 0 1 0 0;
  contrast '1 vs 4' D -1 0 0 1 0;
  contrast '1 vs 5' D -1 0 0 0 1;
  ods output ParameterEstimates=myout;
run; quit;
ods trace off;

***********comparing regression line for the sleep1 data**********;
data sleep;
  set alr3.sleep1;
  logBodyWt=log2(BodyWt);
proc glm data=sleep; *general model;
  class D;
  model TS=D logBodyWt D*logBodyWt / solution;
run; quit;
proc glm data=sleep; *parallel model;
  class D;
  model TS=D logBodyWt /noint;
run; quit;
proc glm data=sleep; *common intercept;
  class D;
  model TS=D*logBodyWt/solution;
run; quit;
proc glm data=sleep; *coincident lines;
  class D;
  model TS=logBodyWt;
run; quit;

*******partial one-dimensional model (POD) for the ais data*******;
proc nlmixed data=alr3.ais;
  parms beta0=1 beta1=1 beta2=1 beta3=1 beta4=1 eta1=1 sig2=1;
  g=beta2*Ht+beta3*Wt+beta4*RCC;
  LBM_exp=beta0+beta1*Sex+g+eta1*Sex*g;
  model LBM~normal(LBM_exp,sig2);
  predict g out=m1;
  predict LBM_exp out=m2;
run; quit;
data m1; set m1; g=Pred; keep LBM g Sex;
data pod; set m1; set m2; LBM_exp=Pred; keep LBM g LBM_exp Sex;
proc sort data=pod; by g;
goptions reset=all;
symbol1 v=point c=black i=join l=1;
symbol2 v=point c=red i=join l=2;
axis2 label=(a=90 'LBM');
axis3 label=(a=90 '');
proc gplot data=pod;
  plot LBM_exp*g=Sex /vaxis=axis2 hminor=0 vminor=0;
  plot2 LBM*g=Sex /vaxis=axis3 vminor=0 nolegend;
run; quit;

**********random coefficient model for the chloride data**********;
proc mixed data=alr3.chloride method=reml covtest;
  class Marsh Type;
  model Cl=Type Month /noint cl;
  random INTERCEPT /type=un subject=Marsh;
run; quit;
proc mixed data=chloride method=reml covtest;
  class Marsh Type;
  model Cl=Type Month /noint cl;
  random Type /type=un subject=Marsh;
run; quit;
