******************************************************************;
*******************Chapter7. Transformations**********************;
******************************************************************;

*****************transforming the predictors only*****************;
data ufcwc;
  set alr3.ufcwc;
  Dbh_tran1=(Dbh**(-1)-(-1))/(-1);
  Dbh_tran2=log(Dbh);
proc reg data=ufcwc;
  model Height=Dbh;
  output out=trans predicted=Hhat;
run;
proc reg data=ufcwc;
  model Height=Dbh_tran1;
  output out=trans1 predicted=Hhat1;
run;
proc reg data=ufcwc;
  model Height=Dbh_tran2;
  output out=trans2 predicted=Hhat2;
run;
data trans; set trans; set trans1; set trans2;
proc sort data=trans; by Dbh;
goptions reset=all;
symbol1 v=circle h=1 c=black;
symbol2 v=point h=1 c=black i=join l=1;
symbol3 v=point h=1 c=black i=join l=2;
symbol4 v=point h=1 c=black i=join l=3;
axis2 label=(a=90 'Height');
legend1 label=none value=(h=1 font=swiss 'Height' '1' '-1' '0')
  position=(top right inside) cborder=black;
proc gplot data=trans;
  plot Height*Dbh=1 Hhat*Dbh=2 Hhat1*Dbh=3 Hhat2*Dbh=4
       /overlay hminor=0 vaxis=axis2 vminor=0 legend=legend1;
run; quit;
*** use nonlinear regression to estimate a transformation;
options nocenter ls=75 pagesize=10000;
proc reg data=alr3.ufcwc;
  model Height = Dbh; 
  run;
proc nlin data=alr3.ufcwc; 
  parms beta0 = 111 beta1=.3 lam=1;
  model Height = beta0 + 
         beta1*( if(abs(lam) < .001) then log(Dbh) else (Dbh**lam -1)/lam);
  output out=nlinout predictor=yhat;
run;
proc sort data=nlinout; by Dbh;
goptions reset=all;
symbol1 v=circle h=1 c=black;
symbol2 v=point h=1 c=black i=join l=1;
proc gplot data=nlinout;
 plot Height*Dbh=1 yhat*Dbh=2/overlay;
run; quit;

****************transforming the response only, Figure 7.2********************;
data highway;
  set alr3.highway;
  logLen=log2(Len); logADT=log2(ADT);
  logTrks=log2(Trks); logSigs1=log2((Len*Sigs+1)/Len);
  Rate_trans1=log(Rate); Rate_trans2=(Rate**(.185)-(.185))/(.185);
proc reg data=highway;
  model Rate=logLen logADT logTrks Slim Shld logSigs1;
  output out=trans predicted=Rhat;
run;
proc reg data=trans;
  model Rhat=Rate_trans1;
  output out=trans1 predicted=Rhat1;
run;
proc reg data=trans;
  model Rhat=Rate_trans2;
  output out=trans2 predicted=Rhat2;
run;
data trans; set trans; set trans1; set trans2;
proc print data=trans; run;
proc sort data=trans; by Rate;
goptions reset=all;
symbol1 v=circle h=1 c=black;
symbol2 v=point h=1 c=black i=join l=1;
symbol3 v=point h=1 c=black i=join l=2;
axis2 label=(a=90 'Predicted Rate');
legend1 label=none value=(h=1 font=swiss 'Untransformed' '0' '0.185')
  position=(top left inside) cborder=black;
proc gplot data=trans;
  plot Rhat*Rate=1 Rhat1*Rate=2 Rhat2*Rate=3
       /overlay hminor=0 vaxis=axis2 vminor=0 legend=legend1;
run; quit;

**********************the Box-Cox method**************************;
data highway;
  set alr3.highway;
  logLen=log2(Len); logADT=log2(ADT);
  logTrks=log2(Trks); logSigs1=log2((Len*Sigs+1)/Len);
proc transreg data=highway;
  model boxcox(Rate /convenient lambda=-1 to 1 by .005)=
        identity(logLen logADT logTrks Slim Shld logSigs1);
run;

************scatterplot matrix for the highway data***************;
proc insight data=alr3.highway;
  scatter Rate Len ADT Trks Slim Shld Sigs
         *Rate Len ADT Trks Slim Shld Sigs;
run;
