******************************************************************;
***********Chapter8. Regression Diagnostics: Residuals************;
******************************************************************;
options nocenter linesize=75 pagesize=1000;
***************computing the leverages for the ufcwc data*********;
proc reg data=alr3.ufcwc;
  model Height=Dbh;
  output out=m1 h=hat;
  print all;
run;

***************the residual plot for the physics data*************;
data physics;
  set alr3.physics;
  w=1/sd**2;
proc reg data=physics;
  model y=x;
  weight w;
  output out=m1 residual=res predicted=yhat h=hat;
run;
data m1; set m1;
pearson_res=sqrt(w)*res; *compute correct residuals;
proc print data=m1; run;
goptions reset=all;
symbol1 v=circle c=black i=none;
symbol2 v=point c=black i=join l=2;
axis2 label=(a=90 'Pearson residual');
proc gplot data=m1;
  plot pearson_res*yhat=1 /vref=0 hminor=0 vaxis=axis2 vminor=0;
run;
quit;

***************linked plots for the fuel2001 data*****************;
data fuel2001;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  Income=Income/1000;
  logMiles=log2(Miles);
proc insight data=fuel2001;
  fit Fuel=Tax Dlic Income logMiles;
  scatter R_Fuel*Tax Dlic Income logMiles P_Fuel /label=State;
  scatter Fuel*P_Fuel /label=State;
run;

************tests for curvature for the fuel2001 data*************;
data fuel2001;
  set alr3.fuel2001;
  Dlic=Drivers*1000/Pop;
  Fuel=FuelC*1000/Pop;
  Income=Income/1000;
  logMiles=log2(Miles);
proc reg data=fuel2001; *this proc reg is simply to retrieve fitted values;
  model Fuel=Tax Dlic Income logMiles /noprint;
  output out=m1 predicted=pred;
run;
data m1; *this data step is to add quadratic terms to the old data set;
  set m1;
  Tax2=Tax**2; Dlic2=Dlic**2;
  Income2=Income**2; logMiles2=logMiles**2;
  pred2=pred**2;
proc reg data=m1; *fit models with one quadratic term added each time;
  model Fuel=Tax Tax2 Dlic Income logMiles;
  model Fuel=Tax Dlic Dlic2 Income logMiles;
  model Fuel=Tax Dlic Income Income2 logMiles;
  model Fuel=Tax Dlic Income logMiles logMiles2;
  model Fuel=Tax Dlic Income logMiles pred2;
  ods output ParameterEstimates=est;
  *save the parameter estimates and t-test results;
run;
data est; set est; Term=scan(Variable,1,'2');
data est; set est; where Term ne Variable;
                   *keep the test statistics for added terms only;
  if Term='pred' then do;
     Probt=2*probnorm(tValue); Term='Tukey test'; end;
proc print data=est; var Term tValue Probt; run;
*final display of T8.1 in the book;

*******test for non-constant variance for the snowgeese data******;
proc reg data=alr3.snowgeese;
  model photo=obs1;
  output out=m1 residual=res predicted=pred;
run;
data ncvartest;
  set m1;
  r2 = res**2;
proc reg data=ncvartest;
  model r2=pred;
run;

proc reg data=alr3.snowgeese;
  model photo=obs1;
  output out=m1 residual=res;
run;
proc iml;
use m1;
read all var {res} where (res^=.) into res;
n=nrow(res);
one=j(n,1,1);
sig2=t(res)*res/n;
U=diag(res)*diag(res)*one/sig2;
create m2 var{U};
append;
close m2;
quit;
data m2; set m1; set m2;
proc reg data=m2;
  model U=obs1;
  ods output Anova=a1;
run;
data ncv;
  set a1; where Source eq 'Model';
  Chisquare=SS/2;
  Df=DF;
  p=1-probchi(Chisquare,Df);
  keep Chisquare Df p;
proc print data=ncv noobs;
  title 'Non-constant Variance Score Test';
run;

******test for non-constant variance for the sniffer data*****;
proc reg data=alr3.sniffer;
  model Y=TankTemp GasTemp TankPres GasPres;
  output out=m1 residual=res;
run;
proc iml;
use m1;
read all var {res} where (res^=.) into res;
n=nrow(res);
one=j(n,1,1);
sig2=t(res)*res/n;
U=diag(res)*diag(res)*one/sig2;
create m2 var{U};
append;
close m2;
quit;
data m2; set m1; set m2;
proc reg data=m2;
  model U=TankTemp GasPres;
  ods output Anova=a1;
run;
data ncv;
  set a1; where Source eq 'Model';
  Chisquare=SS/2;
  Df=DF;
  p=1-probchi(Chisquare,Df);
  keep Chisquare Df p;
proc print data=ncv noobs;
  title 'Non-constant Variance Score Test';
run;

*****************mean checking plot for the ufcwc data****************;
proc loess data=alr3.ufcwc;
  model Height=Dbh /smooth=.6;
  ods output OutputStatistics=m1;
run;
proc reg data=alr3.ufcwc;
  model Height=Dbh;
  output out=m2 predicted=OLS_pred;
run;
data fit; set m1; set m2;
proc sort data=fit; by Dbh;
goptions reset=all;
symbol1 v=circle c=black;
symbol2 v=point c=black i=join l=33;
symbol3 v=point c=black i=join l=1;
axis2 label=(a=90 'Height');
proc gplot data=fit;
  plot DepVar*Dbh=1 OLS_pred*Dbh=2 Pred*Dbh=3
       /overlay hminor=0 vaxis=axis2 vminor=0;
run; quit;

*******mean checking plots (alr{F8.13a-d}) for the UN2 data*******;
proc reg data=alr3.UN2;
  model logFertility=logPPgdp Purban;
  output out=ols predicted=OLS_pred;
run;
goptions reset=all;
symbol1 v=circle h=1 c=black;
symbol2 v=point c=black i=join l=33;
symbol3 v=point c=black i=join l=1;
axis2 label=(a=90 'log(Fertility)');
*------------------------------------------------------;
*BEGIN plotting alr{F8.13A};
proc reg data=ols;
  model OLS_pred=logPPgdp;
  output out=mmp1 predicted=mmp1;
run;
proc loess data=ols;
  model logFertility=logPPgdp /smooth=.67;
  ods output OutputStatistics=loess1;
run;
data fit1; set mmp1; set loess1;
proc sort data=fit1;
  by logPPgdp; *sort by logPPgdp before plotting alr{F8.13A};
proc gplot data=fit1;
  plot DepVar*logPPgdp=1 mmp1*logPPgdp=2 Pred*logPPgdp=3
       /overlay hminor=0 vaxis=axis2 vminor=0;
run;
quit;
*END plotting alr{F8.13A};
*------------------------------------------------------;
*BEGIN plotting alr{F8.13B};
proc reg data=ols;
  model OLS_pred=Purban;
  output out=mmp2 predicted=mmp2;
run;
proc loess data=ols;
  model logFertility=Purban /smooth=.67;
  ods output OutputStatistics=loess2;
run;
data fit2; set mmp2; set loess2;
proc sort data=fit2;
  by Purban; *sort by Purban before plotting alr{F8.13B};
proc gplot data=fit2;
  plot DepVar*Purban=1 mmp2*Purban=2 Pred*Purban=3
       /overlay hminor=0 vaxis=axis2 vminor=0;
run;
quit;
*END plotting alr{F8.13B};
*------------------------------------------------------;
*BEGIN plotting alr{F8.13C};
data mmp3; set ols; mmp3=OLS_pred; FittedValues=OLS_pred;
proc loess data=ols;
  model logFertility=OLS_pred /smooth=.67;
  ods output OutputStatistics=loess3;
run;
data fit3; set mmp3; set loess3;
proc sort data=fit3;
  by FittedValues; *sort by FittedValues before plotting alr{F8.13C};
proc gplot data=fit3;
  plot DepVar*FittedValues=1 mmp3*FittedValues=2 Pred*FittedValues=3
       /overlay hminor=0 vaxis=axis2 vminor=0;
run;
quit;
*END plotting alr{F8.13C};
*------------------------------------------------------;
*BEGIN plotting alr{F8.13D};
data random;
  set ols; a1=ranuni(-1); a2=ranuni(-1);
  RandomDirection=a1*logPPgdp+a2*Purban;
proc reg data=random;
  model OLS_pred=RandomDirection;
  output out=mmp4 predicted=mmp4;
run;
proc loess data=random;
  model logFertility=RandomDirection /smooth=.67;
  ods output OutputStatistics=loess4;
run;
data fit4; set mmp4; set loess4;
proc sort data=fit4; by RandomDirection;
*sort by RandomDirection before plotting alr{F8.13D};
proc gplot data=fit4;
  plot DepVar*RandomDirection=1
       mmp4*RandomDirection=2
       Pred*RandomDirection=3
       /overlay hminor=0 vaxis=axis2 vminor=0;
run;
quit;
*END plotting alr{F8.13D};

*****variance checking plots (alr{F8.15a,b}) for the UN2 data*****;
data UN2;
  set alr3.UN2;
  Purban2=Purban**2;
goptions reset=all;
symbol1 v=circle h=1 c=black;
symbol2 v=point c=black i=join l=33;
symbol3 v=point c=black i=join l=1;
axis1 label=('Fitted values, original model');
axis2 label=('Fitted values, quadratic term added');
axis3 label=(a=90 'log(Fertility)');
*------------------------------------------------------;
*BEGIN plotting alr{F8.15A};
proc reg data=UN2;
  model logFertility=logPPgdp Purban;
  output out=ols1 predicted=OLS_pred;
  ods output Anova=anova1;
run;
data anova1; set anova1; sigma=sqrt(MS); where Source='Error'; keep sigma;
proc sql; create table ols1 as select * from ols1, anova1; quit;
data mmp1; *compute OLS_mean+/-se;
  set ols1; FittedValues=OLS_pred;
  upper1=FittedValues+sigma; lower1=FittedValues-sigma;
proc print data=mmp1; run;
proc loess data=ols1;
  model logFertility=OLS_pred /smooth=.67 residual;
  ods output OutputStatistics=loess1;
run;
data loess1; set loess1; r2=Residual**2; mean_loess=Pred; y=DepVar;
proc loess data=loess1; *following alr{A.5};
  model r2=OLS_pred /smooth=.67;
  ods output OutputStatistics=loess_se1;
run;
data loess1; *compute loess+/-se;
  set loess1; set loess_se1;
  if Pred le 0 then do;
    upper_loess1=mean_loess; lower_loess1=mean_loess; end;
  else do;
  upper_loess1=mean_loess+sqrt(Pred);
  lower_loess1=mean_loess-sqrt(Pred); end;
data fit1; set mmp1; set loess1; *combine data sets to draw alr{F8.15A};
proc sort data=fit1;
  by FittedValues; *sort by FittedValues before plotting;
proc gplot data=fit1;
  plot y*FittedValues=1 (FittedValues upper1 lower1)*FittedValues=2
       (mean_loess upper_loess1 lower_loess1)*FittedValues=3
       /overlay haxis=axis1 hminor=0 vaxis=axis3 vminor=0;
run;
quit;
*END plotting alr{F8.15A};
*------------------------------------------------------;
*BEGIN plotting alr{F8.15B};
proc reg data=UN2;
  model logFertility=logPPgdp Purban Purban2;
  output out=ols2 predicted=OLS_pred;
  ods output Anova=anova2;
run;
data anova2; set anova2; sigma=sqrt(MS); where Source='Error'; keep sigma;
proc sql; create table ols2 as select * from ols2, anova2; quit;
data mmp2; *compute OLS_mean+/-se;
  set ols2; FittedValues=OLS_pred;
  upper2=FittedValues+sigma; lower2=FittedValues-sigma;
proc print data=mmp2; run;
proc loess data=ols2;
  model logFertility=OLS_pred /smooth=.67 residual;
  ods output OutputStatistics=loess2;
run;
data loess2; set loess2; r2=Residual**2; mean_loess=Pred; y=DepVar;
proc loess data=loess2; *following alr{A.5};
  model r2=OLS_pred /smooth=.67;
  ods output OutputStatistics=loess_se2;
run;
data loess2; *compute loess+/-se;
  set loess2; set loess_se2;
  if Pred le 0 then do;
    upper_loess2=mean_loess; lower_loess2=mean_loess; end;
  else do;
  upper_loess2=mean_loess+sqrt(Pred);
  lower_loess2=mean_loess-sqrt(Pred); end;
data fit2; set mmp2; set loess2; *combine data sets to draw alr{F8.15A};
proc sort data=fit2;
  by FittedValues; *sort by FittedValues before plotting;
proc gplot data=fit2;
  plot y*FittedValues=1 (FittedValues upper2 lower2)*FittedValues=2
       (mean_loess upper_loess2 lower_loess2)*FittedValues=3
       /overlay haxis=axis2 hminor=0 vaxis=axis3 vminor=0;
run;
quit;
*END plotting alr{F8.15B};
