******************************************************************;
*****************Chapter9. Outliers and Influence*****************;
******************************************************************;

*****outlier test for the UN2 data with Bonferroni adjustment*****;
options nocenter linesize=75 pagesize=10000;
proc reg data=alr3.UN2;
  model logFertility=logPPgdp Purban;
  output out=m1 rstudent=rstu;
  ods output Anova=anova;
run;
data m1;
  set m1;
  retain casenum 0;
  casenum=casenum+1;
  absrstu=abs(rstu);
proc sort data=m1; by descending absrstu; run;
data m1; set m1 (obs=10); p=min(1,193*2*(1-probt(absrstu,190-1)));
proc print data=m1;run;

*********obtaining influence statistics for the UN2 data**********;
proc reg data=alr3.UN2;
  model logFertility=logPPgdp Purban;
  output out=m1 residual=res rstudent=rstu h=hat cookd=cookd;
run;

******fitting model for the UN2 data with some cases deleted******;
data UN2;
  set alr3.UN2;
  retain casenum 0;
  casenum=casenum+1;
proc reg data=UN2;
  model logFertility=logPPgdp Purban;
run;
reweight casenum=2; refit; print; run; *delete case 2;
reweight undo; *restore the last deleted case;
reweight casenum=1;
reweight casenum=3;
reweight casenum=6;
refit; print; run; *delete case 1,3,6 from the data set;
reweight allobs /reset; *restore all previously deleted cases;

****************Cook's distance for the rat data******************;
goptions reset=all;
proc reg data=alr3.rat;
  model y=BodyWt LiverWt Dose;
  output out=m1 cookd=cookd;
  plot cookd.*obs.;
run;

******************3 index plots for the rat data******************;
proc reg data=alr3.rat;
  model y=BodyWt LiverWt Dose;
  output out=m1 h=hat rstudent=rstu cookd=cookd;
  ods output Anova=anova;
run;
data anova; set anova; df=DF; where Source eq 'Error'; keep df;
proc sql; create table test as select * from m1, anova; quit;
data test; set test; p_val=1-probt(rstu,df);
data index; set m1; set test; retain casenum 0; casenum=casenum+1;
goptions reset=all;
symbol v=circle i=join l=1;
axis2 label=(a=90 'Outlier test (studentized residual)');
axis3 label=(a=90 'Leverage');
axis4 label=(a=90 'Cooks distance');
proc gplot data=index;
  plot rstu*casenum /hminor=0 vaxis=axis2 vminor=0;
  plot hat*casenum /hminor=0 vaxis=axis3 vminor=0;
  plot cookd*casenum /hminor=0 vaxis=axis4 vminor=0;
run;
quit;

*****QQnorm plots for the heights data and the transact data******;
goptions reset=all;
*************************************;
**qqnorm plot for the heights data***;
*************************************;
proc reg data=alr3.heights;
  model Dheight=Mheight;
  title 'QQ Plot';
  plot r.*nqq. /noline mse cframe=ligr;
run;
*************************************;
**qqnorm plot for the transact data**;
*************************************;
proc reg data=alr3.transact;
  model Time=T1 T2;
  title 'QQ Plot';
  plot r.*nqq. /noline mse cframe=ligr;
run;
